# coding: utf-8

"""
Jaqpot API

A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

The version of the OpenAPI document: 1.0.0
Contact: upci.ntua@gmail.com
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class FeatureType(str, Enum):
    """
    FeatureType
    """

    """
    allowed enum values
    """
    INTEGER = "INTEGER"
    FLOAT = "FLOAT"
    CATEGORICAL = "CATEGORICAL"
    SMILES = "SMILES"
    STRING = "STRING"
    TEXT = "TEXT"
    FLOAT_ARRAY = "FLOAT_ARRAY"
    STRING_ARRAY = "STRING_ARRAY"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of FeatureType from a JSON string"""
        return cls(json.loads(json_str))
