# coding: utf-8

"""
Jaqpot API

A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

The version of the OpenAPI document: 1.0.0
Contact: upci.ntua@gmail.com
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ModelVisibility(str, Enum):
    """
    ModelVisibility
    """

    """
    allowed enum values
    """
    PUBLIC = "PUBLIC"
    ORG_SHARED = "ORG_SHARED"
    PRIVATE = "PRIVATE"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ModelVisibility from a JSON string"""
        return cls(json.loads(json_str))
